@@ -15,9 +15,9 @@ use std::cmp::Ordering;
 use std::mem;
 
 #[derive(Debug)]
-pub(crate) struct Terminal {
-    pub cols: usize,
-    pub rows: usize,
+pub struct Terminal {
+    cols: usize,
+    rows: usize,
     buffer: Buffer,
     other_buffer: Buffer,
     active_buffer_type: BufferType,
@@ -40,20 +40,20 @@ pub(crate) struct Terminal {
     xtwinops: bool,
 }
 
-#[derive(Debug, PartialEq)]
-enum BufferType {
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum BufferType {
     Primary,
     Alternate,
 }
 
 #[derive(Debug, Copy, Clone, PartialEq)]
-pub enum CursorKeysMode {
+pub(crate) enum CursorKeysMode {
     Normal,
     Application,
 }
 
 #[derive(Debug, PartialEq)]
-pub struct SavedCtx {
+pub(crate) struct SavedCtx {
     pub cursor_col: usize,
     pub cursor_row: usize,
     pub pen: Pen,
@@ -115,6 +115,14 @@ impl Terminal {
         }
     }
 
+    pub fn size(&self) -> (usize, usize) {
+        (self.cols, self.rows)
+    }
+
+    pub fn active_buffer_type(&self) -> BufferType {
+        self.active_buffer_type
+    }
+
     pub fn execute(&mut self, fun: Function) {
         use Function::*;
 
@@ -617,11 +625,11 @@ impl Terminal {
         }
     }
 
-    pub fn view(&self) -> &[Line] {
+    pub fn view(&self) -> impl Iterator<Item = &Line> {
         self.buffer.view()
     }
 
-    pub fn lines(&self) -> &[Line] {
+    pub fn lines(&self) -> impl Iterator<Item = &Line> {
         self.buffer.lines()
     }
 
@@ -641,7 +649,7 @@ impl Terminal {
     pub fn verify(&self) {
         assert!(self.cursor.row < self.rows);
         assert!(self.cursor.col <= self.cols);
-        assert!(self.lines().iter().all(|line| line.len() == self.cols));
+        assert!(self.lines().all(|line| line.len() == self.cols));
         assert!(!self.lines().last().unwrap().wrapped);
 
         for line in self.lines() {
@@ -675,12 +683,16 @@ impl Terminal {
         assert_eq!(self.bottom_margin, other.bottom_margin);
         assert_eq!(self.saved_ctx, other.saved_ctx);
         assert_eq!(self.alternate_saved_ctx, other.alternate_saved_ctx);
-        assert_eq!(self.primary_buffer().view(), other.primary_buffer().view());
+
+        assert_eq!(
+            self.primary_buffer().view().collect::<Vec<_>>(),
+            other.primary_buffer().view().collect::<Vec<_>>()
+        );
 
         if self.active_buffer_type == BufferType::Alternate {
             assert_eq!(
-                self.alternate_buffer().view(),
-                other.alternate_buffer().view()
+                self.alternate_buffer().view().collect::<Vec<_>>(),
+                other.alternate_buffer().view().collect::<Vec<_>>()
             );
         }
     }
